import os
from openpyxl import Workbook, load_workbook
from openpyxl.styles import Alignment
from openpyxl.utils import get_column_letter


class Excel:
    def __init__(self, filename, sheet_name="sheet1"):
        self.filename = filename
        self.workbook = None
        self.sheet = None
        self.max_column_width = []
        self.sheet_name = sheet_name

    def build(self, sheet):
        if os.path.exists(self.filename):
            self.workbook = load_workbook(self.filename)
        else:
            self.workbook = Workbook()
            del self.workbook["Sheet"]
        if sheet not in [s.title for s in self.workbook]:
            self.sheet = self.workbook.create_sheet(sheet)
        else:
            self.sheet = self.workbook[sheet]

    def write(self, row_header, data):
        max_row = self.sheet.max_row
        for i, rh in enumerate(row_header):
            self.sheet.cell(1, i + 1, rh)
        for i, v in enumerate(data):
            self.sheet.cell(max_row + 1, i + 1, str(v))

    def width_fit(self):
        """
        获取表格每列值的最大长度，并进行设置
        """
        i = 0
        for col in self.sheet.columns:  # 获取每一列
            for height in range(len(col)):  # 某列序号
                if height == 0:
                    self.max_column_width.append(len(str(col[height].value)))  # 将第一列的单元格的长度加入列表
                else:
                    if self.max_column_width[i] < len(str(col[height].value)):  # 获得每列中的内容的最大宽度
                        self.max_column_width[i] = len(str(col[height].value))
            i += 1
        for i in range(len(self.max_column_width)):
            # 根据列的数字返回字母
            col_letter = get_column_letter(i + 1)
            # 当宽度大于100，宽度设置为100
            if self.max_column_width[i] > 100:
                self.sheet.column_dimensions[col_letter].width = 100
            # 只有当宽度大于10，才设置列宽
            else:
                self.sheet.column_dimensions[col_letter].width = self.max_column_width[i] + 2

    def align_center(self):
        nrows = self.sheet.max_row  # 获得行数
        ncols = self.sheet.max_column
        for i in range(nrows):
            for j in range(ncols):
                self.sheet.cell(row=i + 1, column=j + 1).alignment = Alignment(horizontal='center', vertical='center')

    def close(self):
        self.workbook.save(self.filename)
        self.workbook.close()

    def __enter__(self):
        self.build(self.sheet_name)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.width_fit()
        self.align_center()
        self.workbook.save(self.filename)
        self.workbook.close()
        return True


if __name__ == '__main__':
    row_header = ["真空能级", "费米能级", "功函数", "带隙", "CBM", "VBM"]
    data = ["/bin/bash", 3.206, -2.7523, 5.9582999999999995, 2.4653, -3.713399999999999, -6.178699999999999]
    with Excel("1.xlsx", "sheet1") as excel:
        excel.write(row_header, data)