from os.path import join


def write_incar(**kwargs):
    """
    :param kwargs:Required key:path, vasp_args...
    :return:
    """
    for k, v in kwargs.items():
        if isinstance(v, bool):
            if v is True:
                kwargs[k] = ".TRUE."
            elif v is False:
                kwargs[k] = ".FALSE."
        if isinstance(v, (list, tuple)):
            v = [str(i) for i in v]
            kwargs[k] = " ".join(v)
        if type(v) is int or type(v) is float:
            kwargs[k] = str(v)
    path = kwargs.pop("directory")
    with open(path, "w+") as tmp:
        for k, v in kwargs.items():
            tmp.write(f"{k.upper().ljust(8)} = {v}\n")


def write_kpoints(**kwargs):
    """
    default parameters:
    density:20
    mode:line-mode
    coord:Reciprocal
    examples:
        write_kpoints(
            kpaths=["GAMMA", "M", "K", "GAMMA"],
            kpts={"GAMMA": (0.0, 0.0, 0.0),
                  "M": (0.5, 0.0, 0.0),
                  "K": (0.33, 0.33, 0.33)},
            density=20
        )
    """
    data = {}
    kpaths = []
    output = ["K-PATH Generated by ZH", ]
    path = kwargs.pop("directory")
    if "density" in kwargs.keys():
        output.append(str(kwargs["density"]).rjust(5))
    else:
        output.append("20".rjust(5))
    if "mode" in kwargs.keys():
        output.append(kwargs["mode"])
    else:
        output.append("Line-Mode")
    if "coord" in kwargs.keys():
        output.append(kwargs["coord"])
    else:
        output.append("Reciprocal")
    if "kpts" in kwargs.keys():
        for k, v in kwargs["kpts"].items():
            tmp = str(v[0]).ljust(12, "0").rjust(15) + \
                  str(v[1]).ljust(12, "0").rjust(15) + \
                  str(v[2]).ljust(12, "0").rjust(15) + \
                  k.ljust(10).rjust(16)
            data[k] = tmp
    else:
        raise ValueError("Please specify the kpts!!!")
    if "kpaths" in kwargs.keys():
        for index, kpath in enumerate(kwargs["kpaths"]):
            kpaths.append(data[kpath])
            if 0 < index < len(kwargs["kpaths"]) - 1:
                kpaths.append(data[kpath])
    for index, value in enumerate(kpaths):
        if index % 2 == 1:
            kpaths[index] += "\n"
    output += kpaths
    with open(path, mode="w+", newline="\n", encoding="utf-8") as tmp:
        for o in output:
            tmp.write(o + "\n")


def write_optcell(fp, cell_type="3D", a=None, b=None, c=None):
    """
    Help to write the vasp OPTCELL file.
    :param fp:file path
    :param cell_type:calculate cell type.
    :param a:a axis
    :param b:b axis
    :param c:c axis
    """
    if a and b and c:
        _a, _b, _c = a, b, c
    elif cell_type == "1D-a" or cell_type == "1D" or cell_type == "1D-x":
        _a, _b, _c = (1, 1, 1), (0, 0, 0), (0, 0, 0)
    elif cell_type == "1D-b" or cell_type == "1D-y":
        _a, _b, _c = (0, 0, 0), (1, 1, 1), (0, 0, 0)
    elif cell_type == "1D-c" or cell_type == "1D-z":
        _a, _b, _c = (0, 0, 0), (0, 0, 0), (1, 1, 1)
    elif cell_type == "2D":
        _a, _b, _c = (1, 1, 1), (1, 1, 1), (0, 0, 0)
    elif cell_type == "3D":
        _a, _b, _c = (1, 1, 1), (1, 1, 1), (1, 1, 1)
    else:
        raise ValueError(f"{write_optcell.__name__} get wrong args {locals()}")
    _a = list(map(lambda x: str(x), _a))
    _b = list(map(lambda x: str(x), _b))
    _c = list(map(lambda x: str(x), _c))
    fn = join(fp, "OPTCELL")
    with open(fn, "w+") as f:
        s = " ".join(_a) + "\n" + \
            " ".join(_b) + "\n" + \
            " ".join(_c) + "\n"
        f.write(s)


def test():
    write_optcell(".")


if __name__ == '__main__':
    test()
